package de.intevation.mxd.reader;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.ILineSymbol;
import com.esri.arcgis.display.CartographicLineSymbol;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import com.esri.arcgis.display.esriLineCapStyle;
import com.esri.arcgis.display.esriLineJoinStyle;

import org.w3c.dom.Element;
import java.io.IOException;

/**
 * Reads cartoline symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class CartoLineSymbolReader
extends AbstractSymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(CartoLineSymbolReader.class);

    /**
     * Private member.
     */
    private CartographicLineSymbol symbol;

    public CartoLineSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof CartographicLineSymbol) {
            this.symbol = (CartographicLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a CartographicLineSymbol!");
        }
    }


    public CartoLineSymbolReader(ILineSymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof CartographicLineSymbol) {
            this.symbol = (CartographicLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a CartographicLineSymbol!");
        }
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read() {
        logger.debug("read()");
        Element symbolElement = util.addSymbol(parent);

        try {
            symbolElement.setAttribute("name", symbol.getNameString());
        }
        catch(IOException ioe) {
            logger.warn("Could not read name. Setting name to \"default\"");
            symbolElement.setAttribute("name", "default");
        }

        try {
            if(symbol.getColor() instanceof IRgbColor) {
                IRgbColor color = (IRgbColor)symbol.getColor();
                Color c = new Color (
                    color.getRed(),
                    color.getGreen(),
                    color.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            }
            else {
                RgbColor col = new RgbColor();
                col.setRGB(symbol.getColor().getRGB());
                Color c = new Color (
                    col.getRed(),
                    col.getGreen(),
                    col.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            }
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read color." +
                " Setting color to black with no transparency");
            Color black = new Color(0, 0, 0);
            symbolElement.setAttribute("color", String.valueOf(black.getRGB()));
            symbolElement.setAttribute("transparency", "-1");
        }

        int cap;
        try {
            cap = symbol.getCap();
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read line cap." +
                " Setting line cap to \"butt\"");
            cap = -1;
        }
        String capStyle = "butt";
        switch(cap) {
            case esriLineCapStyle.esriLCSRound: capStyle = "round"; break;
            case esriLineCapStyle.esriLCSSquare: capStyle = "square"; break;
            default: break;
        }


        int join;
        try {
            join = symbol.getJoin();
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read line join." +
                " Setting line join to \"bevel\".");
            join = -1;
        }
        String joinStyle = "bevel";
        switch(join) {
            case esriLineJoinStyle.esriLJSRound: joinStyle = "round"; break;
            case esriLineJoinStyle.esriLJSMitre: joinStyle = "miter"; break;
            default: break;
        }
        symbolElement.setAttribute("cap", capStyle);
        symbolElement.setAttribute("join", joinStyle);

        try {
            symbolElement.setAttribute(
                "linestart",
                String.valueOf(symbol.getLineStartOffset()));
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read line start offset." +
                " Setting line start offset to 0.");
            symbolElement.setAttribute("linestart", "0");
        }

        try {
            symbolElement.setAttribute(
                "miterlimit",
                String.valueOf(symbol.getMiterLimit()));
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read miter limit. " +
                " Setting miter limit to 0.");
            symbolElement.setAttribute("miterlimit", "0");
        }

        try {
            symbolElement.setAttribute(
                "offset",
                String.valueOf(symbol.getOffset()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read offset. Setting offset to 0.");
            symbolElement.setAttribute("offset", "0");
        }

        try {
            symbolElement.setAttribute(
                "width",
                String.valueOf(symbol.getWidth()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read width. Setting width to 1.");
            symbolElement.setAttribute("width", "1");
        }
        //TODO Read further attributes depending on the Mapscript functionality:
        //     Template, MarkerSymbol, LineDecorations
        symbolElement.setAttribute("type", "line");
        symbolElement.setAttribute("style", "carto");

        return symbolElement;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
